@@ -344,6 +344,8 @@ clutter_interval_finalize (GObject *gobject)
   g_value_unset (&priv->values[1]);
 
   g_free (priv->values);
+
+  G_OBJECT_CLASS (clutter_interval_parent_class)->finalize (gobject);
 }
 
 static void
@@ -597,7 +599,7 @@ clutter_interval_new_with_values (GType         gtype,
  *
  * Creates a copy of @interval.
  *
- * Return value: the newly created #ClutterInterval
+ * Return value: (transfer full): the newly created #ClutterInterval
  *
  * Since: 1.0
  */
@@ -698,7 +700,7 @@ clutter_interval_set_initial_value (ClutterInterval *interval,
 /**
  * clutter_interval_get_initial_value:
  * @interval: a #ClutterInterval
- * @value: a #GValue
+ * @value: (out caller-allocates): a #GValue
  *
  * Retrieves the initial value of @interval and copies
  * it into @value.
@@ -767,7 +769,7 @@ clutter_interval_set_final_value (ClutterInterval *interval,
 /**
  * clutter_interval_get_final_value:
  * @interval: a #ClutterInterval
- * @value: a #GValue
+ * @value: (out caller-allocates): a #GValue
  *
  * Retrieves the final value of @interval and copies
  * it into @value.
@@ -901,7 +903,7 @@ clutter_interval_validate (ClutterInterval *interval,
  * clutter_interval_compute_value:
  * @interval: a #ClutterInterval
  * @factor: the progress factor, between 0 and 1
- * @value: return location for an initialized #GValue
+ * @value: (out caller-allocates): return location for an initialized #GValue
  *
  * Computes the value between the @interval boundaries given the
  * progress @factor and copies it into @value.
@@ -968,7 +970,7 @@ clutter_interval_compute (ClutterInterval *interval,
 }
 
 /**
- * clutter_interval_register_progress_func:
+ * clutter_interval_register_progress_func: (skip)
  * @value_type: a #GType
  * @func: a #ClutterProgressFunc, or %NULL to unset a previously
  *   set progress function
