@@ -24,15 +24,14 @@
 
 /**
  * SECTION:clutter-shader-effect
- * @short_description: Base class for fragment shader effects
+ * @short_description: Base class for shader effects
  * @See_Also: #ClutterEffect, #ClutterOffscreenEffect
  *
  * #ClutterShaderEffect is an abstract class that implements all the
- * plumbing for creating #ClutterEffect<!-- -->s using GLSL fragment
- * shaders.
+ * plumbing for creating #ClutterEffect<!-- -->s using GLSL shaders.
  *
  * #ClutterShaderEffect creates an offscreen buffer and then applies the GLSL
- * fragment shader (after checking whether the compilation and linking were
+ * shader (after checking whether the compilation and linking were
  * successfull) to the buffer before painting it on screen.
  *
  * <refsect2 id="ClutterShaderEffect-implementing">
@@ -183,26 +182,6 @@ clutter_shader_effect_clear (ClutterShaderEffect *self,
   priv->source_set = FALSE;
 }
 
-static void
-clutter_shader_effect_reset_uniforms (ClutterShaderEffect *effect)
-{
-  ClutterShaderEffectPrivate *priv = effect->priv;
-  GHashTableIter iter;
-  gpointer key, value;
-
-  if (priv->uniforms == NULL)
-    return;
-
-  key = value = NULL;
-  g_hash_table_iter_init (&iter, priv->uniforms);
-  while (g_hash_table_iter_next (&iter, &key, &value))
-    {
-      ShaderUniform *uniform = value;
-
-      uniform->location = -1;
-    }
-}
-
 static void
 clutter_shader_effect_update_uniforms (ClutterShaderEffect *effect)
 {
@@ -298,9 +277,6 @@ clutter_shader_effect_set_actor (ClutterActorMeta *meta,
       return;
     }
 
-  clutter_shader_effect_clear (self, FALSE);
-  clutter_shader_effect_reset_uniforms (self);
-
   parent = CLUTTER_ACTOR_META_CLASS (clutter_shader_effect_parent_class);
   parent->set_actor (meta, actor);
 
@@ -311,25 +287,6 @@ clutter_shader_effect_set_actor (ClutterActorMeta *meta,
 
   CLUTTER_NOTE (SHADER, "Preparing shader effect of type '%s'",
                 G_OBJECT_TYPE_NAME (meta));
-
-  priv->program = cogl_create_program ();
-
-  switch (priv->shader_type)
-    {
-    case CLUTTER_FRAGMENT_SHADER:
-      priv->shader = cogl_create_shader (COGL_SHADER_TYPE_FRAGMENT);
-      break;
-
-    case CLUTTER_VERTEX_SHADER:
-      priv->shader = cogl_create_shader (COGL_SHADER_TYPE_VERTEX);
-      break;
-
-    default:
-      priv->shader = COGL_INVALID_HANDLE;
-      break;
-    }
-
-  g_assert (priv->shader != COGL_INVALID_HANDLE);
 }
 
 static void
@@ -531,11 +488,10 @@ shader_uniform_new (const gchar  *name,
 {
   ShaderUniform *retval;
 
-  retval = g_slice_new (ShaderUniform);
+  retval = g_slice_new0 (ShaderUniform);
   retval->name = g_strdup (name);
   retval->type = G_VALUE_TYPE (value);
   retval->location = -1;
-  retval->value = (GValue){0};
 
   g_value_init (&retval->value, retval->type);
   g_value_copy (value, &retval->value);
@@ -846,8 +802,28 @@ clutter_shader_effect_set_shader_source (ClutterShaderEffect *effect,
   if (priv->source_set)
     return TRUE;
 
+  if (priv->program == COGL_INVALID_HANDLE)
+    priv->program = cogl_create_program ();
+
   if (priv->shader == COGL_INVALID_HANDLE)
-    return FALSE;
+    {
+      switch (priv->shader_type)
+        {
+        case CLUTTER_FRAGMENT_SHADER:
+          priv->shader = cogl_create_shader (COGL_SHADER_TYPE_FRAGMENT);
+          break;
+
+        case CLUTTER_VERTEX_SHADER:
+          priv->shader = cogl_create_shader (COGL_SHADER_TYPE_VERTEX);
+          break;
+
+        default:
+          priv->shader = COGL_INVALID_HANDLE;
+          break;
+        }
+    }
+
+  g_assert (priv->shader != COGL_INVALID_HANDLE);
 
   cogl_shader_source (priv->shader, source);
 
