@@ -325,16 +325,16 @@ _cogl_framebuffer_init_bits (CoglFramebuffer *framebuffer)
 }
 
 static gboolean
-try_creating_fbo (CoglOffscreen *offscreen,
-                  TryFBOFlags flags,
-                  CoglHandle texture)
+try_creating_fbo (TryFBOFlags flags,
+                  CoglHandle texture,
+		  GSList **renderbuffers_return,
+		  GLuint *fbo_handle_return)
 {
   GLuint gl_depth_stencil_handle;
   GLuint gl_depth_handle;
   GLuint gl_stencil_handle;
   GLuint tex_gl_handle;
   GLenum tex_gl_target;
-  GLuint fbo_gl_handle;
   GLenum status;
 
   _COGL_GET_CONTEXT (ctx, FALSE);
@@ -357,9 +357,8 @@ try_creating_fbo (CoglOffscreen *offscreen,
   ctx->dirty_bound_framebuffer = 1;
 
   /* Generate framebuffer */
-  glGenFramebuffers (1, &fbo_gl_handle);
-  GE (glBindFramebuffer (GL_FRAMEBUFFER, fbo_gl_handle));
-  offscreen->fbo_handle = fbo_gl_handle;
+  glGenFramebuffers (1, fbo_handle_return);
+  GE (glBindFramebuffer (GL_FRAMEBUFFER, *fbo_handle_return));
 
   GE (glFramebufferTexture2D (GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
 			      tex_gl_target, tex_gl_handle, 0));
@@ -379,8 +378,8 @@ try_creating_fbo (CoglOffscreen *offscreen,
       GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
                                      GL_DEPTH_ATTACHMENT,
                                      GL_RENDERBUFFER, gl_depth_stencil_handle));
-      offscreen->renderbuffers =
-        g_slist_prepend (offscreen->renderbuffers,
+      *renderbuffers_return =
+        g_slist_prepend (*renderbuffers_return,
                          GUINT_TO_POINTER (gl_depth_stencil_handle));
     }
 
@@ -397,8 +396,8 @@ try_creating_fbo (CoglOffscreen *offscreen,
       GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
                                      GL_DEPTH_ATTACHMENT,
                                      GL_RENDERBUFFER, gl_depth_handle));
-      offscreen->renderbuffers =
-        g_slist_prepend (offscreen->renderbuffers,
+      *renderbuffers_return =
+        g_slist_prepend (*renderbuffers_return,
                          GUINT_TO_POINTER (gl_depth_handle));
     }
 
@@ -413,8 +412,8 @@ try_creating_fbo (CoglOffscreen *offscreen,
       GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
                                      GL_STENCIL_ATTACHMENT,
                                      GL_RENDERBUFFER, gl_stencil_handle));
-      offscreen->renderbuffers =
-        g_slist_prepend (offscreen->renderbuffers,
+      *renderbuffers_return =
+        g_slist_prepend (*renderbuffers_return,
                          GUINT_TO_POINTER (gl_stencil_handle));
     }
 
@@ -425,25 +424,61 @@ try_creating_fbo (CoglOffscreen *offscreen,
     {
       GSList *l;
 
-      GE (glDeleteFramebuffers (1, &fbo_gl_handle));
+      GE (glDeleteFramebuffers (1, fbo_handle_return));
 
-      for (l = offscreen->renderbuffers; l; l = l->next)
+      for (l = *renderbuffers_return; l; l = l->next)
         {
           GLuint renderbuffer = GPOINTER_TO_UINT (l->data);
           GE (glDeleteRenderbuffers (1, &renderbuffer));
         }
+
+      g_slist_free (*renderbuffers_return);
+      *renderbuffers_return = NULL;
+
       return FALSE;
     }
 
   return TRUE;
 }
 
+static gboolean
+try_creating_fbo_alternatives (CoglHandle texhandle,
+			       GSList **renderbuffers_return,
+			       GLuint *fbo_handle_return)
+{
+  static TryFBOFlags  flags;
+  static gboolean     have_working_flags = FALSE;
+
+  if ((have_working_flags &&
+       try_creating_fbo (flags, texhandle,
+                         renderbuffers_return, fbo_handle_return)) ||
+#ifdef HAVE_COGL_GL
+      try_creating_fbo (flags = _TRY_DEPTH_STENCIL, texhandle,
+                        renderbuffers_return, fbo_handle_return) ||
+#endif
+      try_creating_fbo (flags = _TRY_DEPTH | _TRY_STENCIL, texhandle,
+                        renderbuffers_return, fbo_handle_return) ||
+      try_creating_fbo (flags = _TRY_STENCIL, texhandle,
+                        renderbuffers_return, fbo_handle_return) ||
+      try_creating_fbo (flags = _TRY_DEPTH, texhandle,
+                        renderbuffers_return, fbo_handle_return) ||
+      try_creating_fbo (flags = 0, texhandle,
+                        renderbuffers_return, fbo_handle_return))
+    {
+      /* Record that the last set of flags succeeded so that we can
+         try that set first next time */
+      have_working_flags = TRUE;
+
+      return TRUE;
+    }
+
+  return TRUE;
+}
+
 CoglHandle
 cogl_offscreen_new_to_texture (CoglHandle texhandle)
 {
   CoglOffscreen      *offscreen;
-  static TryFBOFlags  flags;
-  static gboolean     have_working_flags = FALSE;
 
   _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
 
@@ -471,20 +506,11 @@ cogl_offscreen_new_to_texture (CoglHandle texhandle)
   _cogl_texture_set_filters (texhandle, GL_NEAREST, GL_NEAREST);
 
   offscreen = g_new0 (CoglOffscreen, 1);
-  offscreen->texture = cogl_handle_ref (texhandle);
 
-  if ((have_working_flags &&
-       try_creating_fbo (offscreen, flags, texhandle)) ||
-      try_creating_fbo (offscreen, flags = _TRY_DEPTH_STENCIL, texhandle) ||
-      try_creating_fbo (offscreen, flags = _TRY_DEPTH | _TRY_STENCIL,
-                        texhandle) ||
-      try_creating_fbo (offscreen, flags = _TRY_STENCIL, texhandle) ||
-      try_creating_fbo (offscreen, flags = _TRY_DEPTH, texhandle) ||
-      try_creating_fbo (offscreen, flags = 0, texhandle))
+  if (try_creating_fbo_alternatives (texhandle, &offscreen->renderbuffers,
+                                     &offscreen->fbo_handle))
     {
-      /* Record that the last set of flags succeeded so that we can
-         try that set first next time */
-      have_working_flags = TRUE;
+      offscreen->texture = cogl_handle_ref (texhandle);
 
       _cogl_framebuffer_init (COGL_FRAMEBUFFER (offscreen),
                               COGL_FRAMEBUFFER_TYPE_OFFSCREEN,
@@ -503,15 +529,12 @@ cogl_offscreen_new_to_texture (CoglHandle texhandle)
 }
 
 static void
-_cogl_offscreen_free (CoglOffscreen *offscreen)
+free_buffers (CoglOffscreen *offscreen)
 {
   GSList *l;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  /* Chain up to parent */
-  _cogl_framebuffer_free (COGL_FRAMEBUFFER (offscreen));
-
   for (l = offscreen->renderbuffers; l; l = l->next)
     {
       GLuint renderbuffer = GPOINTER_TO_UINT (l->data);
@@ -523,10 +546,62 @@ _cogl_offscreen_free (CoglOffscreen *offscreen)
 
   if (offscreen->texture != COGL_INVALID_HANDLE)
     cogl_handle_unref (offscreen->texture);
+}
+
+static void
+_cogl_offscreen_free (CoglOffscreen *offscreen)
+{
+  /* Chain up to parent */
+  _cogl_framebuffer_free (COGL_FRAMEBUFFER (offscreen));
+
+  free_buffers (offscreen);
 
   g_free (offscreen);
 }
 
+gboolean
+cogl_offscreen_set_to_texture (CoglHandle handle,
+                               CoglHandle texhandle)
+{
+  GLuint fbo_handle;
+  GSList *renderbuffers = NULL;
+  
+  if (!cogl_is_offscreen(handle))
+    return FALSE;
+
+  if (!cogl_is_texture (texhandle))
+    return FALSE;
+
+  if (cogl_texture_is_sliced (texhandle))
+    return FALSE;
+
+  _cogl_texture_set_filters (texhandle, GL_NEAREST, GL_NEAREST);
+
+  if (try_creating_fbo_alternatives (texhandle, &renderbuffers, &fbo_handle))
+    {
+      CoglOffscreen *offscreen;
+      CoglFramebuffer *framebuffer;
+    
+      offscreen = _cogl_offscreen_pointer_from_handle(handle);
+      framebuffer = COGL_FRAMEBUFFER (offscreen);
+    
+      free_buffers (offscreen);
+      
+      offscreen->texture = cogl_handle_ref (texhandle);
+      framebuffer->width = cogl_texture_get_width (texhandle);
+      framebuffer->height = cogl_texture_get_height (texhandle);
+
+      offscreen->fbo_handle = fbo_handle;
+      offscreen->renderbuffers = renderbuffers;
+
+      framebuffer->dirty_bitmasks = TRUE;
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 CoglHandle
 _cogl_onscreen_new (void)
 {
@@ -705,7 +780,7 @@ cogl_pop_framebuffer (void)
 
   cogl_object_unref (to_pop);
   ctx->framebuffer_stack =
-    g_slist_remove_link (ctx->framebuffer_stack,
+    g_slist_delete_link (ctx->framebuffer_stack,
                          ctx->framebuffer_stack);
 
   /* If the framebuffer has changed as a result of popping the top
