@@ -3,9 +3,8 @@
  *
  * An OpenGL based 'interactive canvas' library.
  *
- * Authored By Johan Bilien  <johan.bilien@nokia.com>
- *
  * Copyright (C) 2007 OpenedHand
+ * Copyright (C) 2010 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,11 +19,14 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  *
- *
+ * Authors:
+ *  Johan Bilien   <johan.bilien@nokia.com>
+ *  Neil Roberts   <neil@linux.intel.com>
  */
 
 /**
  * SECTION:clutter-x11-texture-pixmap
+ * @Title: ClutterX11TexturePixmap
  * @short_description: A texture which displays the content of an X Pixmap.
  *
  * #ClutterX11TexturePixmap is a class for displaying the content of an
@@ -47,15 +49,11 @@
 #include "clutter-private.h"
 
 #include "cogl/cogl.h"
+#include "cogl/winsys/cogl-texture-pixmap-x11.h"
 
 #include <X11/extensions/Xdamage.h>
 #include <X11/extensions/Xcomposite.h>
 
-#include <sys/ipc.h>
-#include <sys/shm.h>
-#include <sys/time.h>
-#include <X11/extensions/XShm.h>
-
 enum
 {
   PROP_PIXMAP = 1,
@@ -76,6 +74,7 @@ enum
 {
   UPDATE_AREA,
   QUEUE_DAMAGE_REDRAW,
+
   /* FIXME: Pixmap lost signal? */
   LAST_SIGNAL
 };
@@ -103,28 +102,22 @@ struct _ClutterX11TexturePixmapPrivate
   guint         pixmap_width, pixmap_height;
   guint         depth;
 
-  XImage       *image;
-  XShmSegmentInfo shminfo;
-
-  gboolean      automatic_updates;
   Damage        damage;
-  Drawable      damage_drawable;
 
-  /* FIXME: lots of gbooleans. coalesce into bitfields */
-  gboolean	have_shm;
-  gboolean      window_redirect_automatic;
-  gboolean      window_mapped;
-  gboolean      destroyed;
-  gboolean      owns_pixmap;
-  gboolean      override_redirect;
   gint          window_x, window_y;
+
+  guint window_redirect_automatic : 1;
+  guint window_mapped             : 1;
+  guint destroyed                 : 1;
+  guint owns_pixmap               : 1;
+  guint override_redirect         : 1;
+  guint automatic_updates         : 1;
 };
 
 static int _damage_event_base = 0;
 
-/* FIXME: Ultimatly with current cogl we should subclass clutter actor */
-G_DEFINE_TYPE (ClutterX11TexturePixmap, \
-               clutter_x11_texture_pixmap, \
+G_DEFINE_TYPE (ClutterX11TexturePixmap,
+               clutter_x11_texture_pixmap,
                CLUTTER_TYPE_TEXTURE);
 
 static gboolean
@@ -152,150 +145,21 @@ check_extensions (ClutterX11TexturePixmap *texture)
 }
 
 static void
-free_shm_resources (ClutterX11TexturePixmap *texture)
+process_damage_event (ClutterX11TexturePixmap *texture,
+                      XDamageNotifyEvent *damage_event)
 {
-  ClutterX11TexturePixmapPrivate *priv;
-
-  priv = texture->priv;
-
-  if (priv->shminfo.shmid != -1)
-    {
-      XShmDetach(clutter_x11_get_default_display(),
-		 &priv->shminfo);
-      shmdt(priv->shminfo.shmaddr);
-      shmctl(priv->shminfo.shmid, IPC_RMID, 0);
-      priv->shminfo.shmid = -1;
-    }
-}
-
-/* Tries to allocate enough shared mem to handle a full size
- * update size of the X Pixmap. */
-static gboolean
-try_alloc_shm (ClutterX11TexturePixmap *texture)
-{
-  ClutterX11TexturePixmapPrivate  *priv;
-  XImage			  *dummy_image;
-  Display			  *dpy;
-
-  priv = texture->priv;
-  dpy  = clutter_x11_get_default_display();
-
-  g_return_val_if_fail (priv->pixmap, FALSE);
-
-  if (!XShmQueryExtension(dpy) || g_getenv("CLUTTER_X11_NO_SHM"))
-    {
-      priv->have_shm = FALSE;
-      return FALSE;
-    }
-
-  clutter_x11_trap_x_errors ();
-
-  /* We are creating a dummy_image so we can have Xlib calculate
-   * image->bytes_per_line - including any magic padding it may
-   * want - for the largest possible ximage we might need to use
-   * when handling updates to the texture.
-   *
-   * Note: we pass a NULL shminfo here, but that has no bearing
-   * on the setup of the XImage, except that ximage->obdata will
-   * == NULL.
-   */
-  dummy_image =
-    XShmCreateImage(dpy,
-		    DefaultVisual(dpy,
-				  clutter_x11_get_default_screen()),
-		    priv->depth,
-		    ZPixmap,
-		    NULL,
-		    NULL, /* shminfo, */
-		    priv->pixmap_width,
-		    priv->pixmap_height);
-  if (!dummy_image)
-    goto failed_image_create;
-
-  priv->shminfo.shmid = shmget (IPC_PRIVATE,
-				dummy_image->bytes_per_line
-				* dummy_image->height,
-				IPC_CREAT | 0777);
-  if (priv->shminfo.shmid == -1)
-    goto failed_shmget;
-
-  priv->shminfo.shmaddr = shmat (priv->shminfo.shmid, 0, 0);
-  if (priv->shminfo.shmaddr == (void *) -1)
-    goto failed_shmat;
-
-  priv->shminfo.readOnly = False;
-
-  if (XShmAttach (dpy, &priv->shminfo) == 0)
-    goto failed_xshmattach;
-
-  if (clutter_x11_untrap_x_errors ())
-    g_warning ("X Error: Failed to setup XShm");
-
-  XDestroyImage (dummy_image);
-
-  priv->have_shm = TRUE;
-
-  return TRUE;
-
-failed_xshmattach:
-  g_warning ("XShmAttach failed");
-  shmdt (priv->shminfo.shmaddr);
-
-failed_shmat:
-  g_warning ("shmat failed");
-  shmctl (priv->shminfo.shmid, IPC_RMID, 0);
-
-failed_shmget:
-  g_warning ("shmget failed");
-  XDestroyImage (dummy_image);
-
-failed_image_create:
-  if (clutter_x11_untrap_x_errors ())
-    g_warning ("X Error: Failed to setup XShm");
-
-  priv->have_shm = FALSE;
-  return FALSE;
-}
-
-static void
-check_for_pixmap_damage (ClutterX11TexturePixmap *texture)
-{
-  ClutterX11TexturePixmapPrivate *priv = texture->priv;
   Display *dpy;
-  XserverRegion parts;
-  int i, r_count;
-  XRectangle *r_damage;
-  XRectangle r_bounds;
 
-  clutter_x11_trap_x_errors ();
-
-  /*
-   * Retrieve the damaged region and break it down into individual
-   * rectangles so we do not have to update the whole shebang.
-   */
   dpy = clutter_x11_get_default_display();
-  parts = XFixesCreateRegion (dpy, 0, 0);
-  XDamageSubtract (dpy, priv->damage, None, parts);
 
-  r_damage = XFixesFetchRegionAndBounds (dpy,
-                                         parts,
-                                         &r_count,
-                                         &r_bounds);
-
-  clutter_x11_untrap_x_errors ();
-
-  if (r_damage)
-    {
-      for (i = 0; i < r_count; ++i)
-        clutter_x11_texture_pixmap_update_area (texture,
-                                                r_damage[i].x,
-                                                r_damage[i].y,
-                                                r_damage[i].width,
-                                                r_damage[i].height);
-      XFree (r_damage);
-    }
-
-  XFixesDestroyRegion (dpy, parts);
+  /* Cogl will deal with updating the texture and subtracting from the
+     damage region so we only need to queue a redraw */
+  g_signal_emit (texture, signals[QUEUE_DAMAGE_REDRAW],
+                 0,
+                 damage_event->area.x,
+                 damage_event->area.y,
+                 damage_event->area.width,
+                 damage_event->area.height);
 }
 
 static ClutterX11FilterReturn
@@ -315,10 +179,10 @@ on_x_event_filter (XEvent *xev, ClutterEvent *cev, gpointer data)
     {
       XDamageNotifyEvent *dev = (XDamageNotifyEvent*)xev;
 
-      if (dev->drawable != priv->damage_drawable)
+      if (dev->damage != priv->damage)
         return CLUTTER_X11_FILTER_CONTINUE;
 
-      check_for_pixmap_damage (texture);
+      process_damage_event (texture, dev);
     }
 
   return  CLUTTER_X11_FILTER_CONTINUE;
@@ -358,6 +222,32 @@ on_x_event_filter_too (XEvent *xev, ClutterEvent *cev, gpointer data)
   return CLUTTER_X11_FILTER_CONTINUE;
 }
 
+static void
+update_pixmap_damage_object (ClutterX11TexturePixmap *texture)
+{
+  ClutterX11TexturePixmapPrivate *priv = texture->priv;
+  CoglHandle cogl_texture;
+
+  /* If we already have a CoglTexturePixmapX11 then update its
+     damage object */
+  cogl_texture =
+    clutter_texture_get_cogl_texture (CLUTTER_TEXTURE (texture));
+
+  if (cogl_texture && cogl_is_texture_pixmap_x11 (cogl_texture))
+    {
+      if (priv->damage)
+        {
+          const CoglTexturePixmapX11ReportLevel report_level =
+            COGL_TEXTURE_PIXMAP_X11_DAMAGE_BOUNDING_BOX;
+          cogl_texture_pixmap_x11_set_damage_object (cogl_texture,
+                                                     priv->damage,
+                                                     report_level);
+        }
+      else
+        cogl_texture_pixmap_x11_set_damage_object (cogl_texture, 0, 0);
+    }
+}
+
 static void
 create_damage_resources (ClutterX11TexturePixmap *texture)
 {
@@ -367,19 +257,14 @@ create_damage_resources (ClutterX11TexturePixmap *texture)
   priv = texture->priv;
   dpy = clutter_x11_get_default_display();
 
-  if (!priv->window && !priv->pixmap)
+  if (!priv->pixmap)
     return;
 
   clutter_x11_trap_x_errors ();
 
-  if (priv->window)
-    priv->damage_drawable = priv->window;
-  else
-    priv->damage_drawable = priv->pixmap;
-
   priv->damage = XDamageCreate (dpy,
-				priv->damage_drawable,
-				XDamageReportNonEmpty);
+                                priv->pixmap,
+                                XDamageReportBoundingBox);
 
   /* Errors here might occur if the window is already destroyed, we
    * simply skip processing damage and assume that the texture pixmap
@@ -389,7 +274,11 @@ create_damage_resources (ClutterX11TexturePixmap *texture)
   clutter_x11_untrap_x_errors ();
 
   if (priv->damage)
-    clutter_x11_add_filter (on_x_event_filter, (gpointer)texture);
+    {
+      clutter_x11_add_filter (on_x_event_filter, (gpointer)texture);
+
+      update_pixmap_damage_object (texture);
+    }
 }
 
 static void
@@ -408,9 +297,10 @@ free_damage_resources (ClutterX11TexturePixmap *texture)
       XSync (dpy, FALSE);
       clutter_x11_untrap_x_errors ();
       priv->damage = None;
-      priv->damage_drawable = None;
 
       clutter_x11_remove_filter (on_x_event_filter, (gpointer)texture);
+
+      update_pixmap_damage_object (texture);
     }
 }
 
@@ -422,10 +312,9 @@ clutter_x11_texture_pixmap_real_queue_damage_redraw (
                                               gint width,
                                               gint height)
 {
+  ClutterX11TexturePixmapPrivate *priv = texture->priv;
   ClutterActor    *self = CLUTTER_ACTOR (texture);
   ClutterActorBox  allocation;
-  guint	           pixmap_width = 0;
-  guint            pixmap_height = 0;
   float            scale_x;
   float            scale_y;
   ClutterActorBox  clip;
@@ -441,21 +330,19 @@ clutter_x11_texture_pixmap_real_queue_damage_redraw (
    * the whole stage anyways in that case, so just go ahead and do
    * it here.
    */
-  if (!_clutter_actor_has_allocation (self))
+  if (!clutter_actor_has_allocation (self))
     {
       clutter_actor_queue_redraw (self);
       return;
     }
 
+  if (priv->pixmap_width == 0 || priv->pixmap_height == 0)
+    return;
+
   clutter_actor_get_allocation_box (self, &allocation);
 
-  g_object_get (self,
-                "pixmap-width",  &pixmap_width,
-                "pixmap-height", &pixmap_height,
-                NULL);
-
-  scale_x = (allocation.x2 - allocation.x1) / pixmap_width;
-  scale_y = (allocation.y2 - allocation.y1) / pixmap_height;
+  scale_x = (allocation.x2 - allocation.x1) / priv->pixmap_width;
+  scale_y = (allocation.y2 - allocation.y1) / priv->pixmap_height;
 
   clip.x1 = x * scale_x;
   clip.y1 = y * scale_y;
@@ -482,15 +369,12 @@ clutter_x11_texture_pixmap_init (ClutterX11TexturePixmap *self)
       */
     }
 
-  self->priv->image = NULL;
   self->priv->automatic_updates = FALSE;
   self->priv->damage = None;
-  self->priv->damage_drawable = None;
   self->priv->window = None;
   self->priv->pixmap = None;
   self->priv->pixmap_height = 0;
   self->priv->pixmap_width = 0;
-  self->priv->shminfo.shmid = -1;
   self->priv->window_redirect_automatic = TRUE;
   self->priv->window_mapped = FALSE;
   self->priv->destroyed = FALSE;
@@ -503,25 +387,11 @@ static void
 clutter_x11_texture_pixmap_dispose (GObject *object)
 {
   ClutterX11TexturePixmap *texture = CLUTTER_X11_TEXTURE_PIXMAP (object);
-  ClutterX11TexturePixmapPrivate *priv = texture->priv;
 
   free_damage_resources (texture);
 
   clutter_x11_remove_filter (on_x_event_filter_too, (gpointer)texture);
-
-  if (priv->owns_pixmap && priv->pixmap)
-    {
-      XFreePixmap (clutter_x11_get_default_display (), priv->pixmap);
-      priv->pixmap = None;
-    }
-
-  if (priv->image)
-    {
-      XDestroyImage (priv->image);
-      priv->image = NULL;
-    }
-
-  free_shm_resources (texture);
+  clutter_x11_texture_pixmap_set_pixmap (texture, None);
 
   G_OBJECT_CLASS (clutter_x11_texture_pixmap_parent_class)->dispose (object);
 }
@@ -621,26 +491,10 @@ clutter_x11_texture_pixmap_get_property (GObject      *object,
     }
 }
 
-static void
-clutter_x11_texture_pixmap_realize (ClutterActor *actor)
-{
-  ClutterX11TexturePixmap        *texture = CLUTTER_X11_TEXTURE_PIXMAP (actor);
-  ClutterX11TexturePixmapPrivate *priv = texture->priv;
-
-  CLUTTER_ACTOR_CLASS (clutter_x11_texture_pixmap_parent_class)->
-      realize (actor);
-
-  clutter_x11_texture_pixmap_update_area_real (texture,
-					       0, 0,
-					       priv->pixmap_width,
-					       priv->pixmap_height);
-}
-
 static void
 clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
 {
   GObjectClass      *object_class = G_OBJECT_CLASS (klass);
-  ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
   GParamSpec        *pspec;
   ClutterBackend    *default_backend;
 
@@ -650,13 +504,11 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
   object_class->set_property = clutter_x11_texture_pixmap_set_property;
   object_class->get_property = clutter_x11_texture_pixmap_get_property;
 
-  actor_class->realize       = clutter_x11_texture_pixmap_realize;
-
   klass->update_area         = clutter_x11_texture_pixmap_update_area_real;
 
   pspec = g_param_spec_ulong ("pixmap",
-			      "Pixmap",
-			      "The X11 Pixmap to be bound",
+			      P_("Pixmap"),
+			      P_("The X11 Pixmap to be bound"),
 			      0, G_MAXULONG,
 			      None,
 			      G_PARAM_READWRITE);
@@ -664,9 +516,8 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
   g_object_class_install_property (object_class, PROP_PIXMAP, pspec);
 
   pspec = g_param_spec_uint ("pixmap-width",
-                             "Pixmap width",
-                             "The width of the "
-                             "pixmap bound to this texture",
+                             P_("Pixmap width"),
+                             P_("The width of the pixmap bound to this texture"),
                              0, G_MAXUINT,
                              0,
                              G_PARAM_READABLE);
@@ -674,9 +525,8 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
   g_object_class_install_property (object_class, PROP_PIXMAP_WIDTH, pspec);
 
   pspec = g_param_spec_uint ("pixmap-height",
-                             "Pixmap height",
-                             "The height of the "
-                             "pixmap bound to this texture",
+                             P_("Pixmap height"),
+                             P_("The height of the pixmap bound to this texture"),
                              0, G_MAXUINT,
                              0,
                              G_PARAM_READABLE);
@@ -684,9 +534,8 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
   g_object_class_install_property (object_class, PROP_PIXMAP_HEIGHT, pspec);
 
   pspec = g_param_spec_uint ("pixmap-depth",
-                             "Pixmap Depth",
-                             "The depth (in number of bits) of the "
-                             "pixmap bound to this texture",
+                             P_("Pixmap Depth"),
+                             P_("The depth (in number of bits) of the pixmap bound to this texture"),
                              0, G_MAXUINT,
                              0,
                              G_PARAM_READABLE);
@@ -694,17 +543,17 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
   g_object_class_install_property (object_class, PROP_DEPTH, pspec);
 
   pspec = g_param_spec_boolean ("automatic-updates",
-                                "Automatic Updates",
-                                "If the texture should be kept in "
-                                "sync with any pixmap changes.",
+                                P_("Automatic Updates"),
+                                P_("If the texture should be kept in "
+                                   "sync with any pixmap changes."),
                                 FALSE,
                                 G_PARAM_READWRITE);
 
   g_object_class_install_property (object_class, PROP_AUTO, pspec);
 
   pspec = g_param_spec_ulong ("window",
-			      "Window",
-			      "The X11 Window to be bound",
+			      P_("Window"),
+			      P_("The X11 Window to be bound"),
 			      0, G_MAXULONG,
 			      None,
 			      G_PARAM_READWRITE);
@@ -712,9 +561,9 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
   g_object_class_install_property (object_class, PROP_WINDOW, pspec);
 
   pspec = g_param_spec_boolean ("window-redirect-automatic",
-                                "Window Redirect Automatic",
-                                "If composite window redirects are set to "
-                                "Automatic (or Manual if false)",
+                                P_("Window Redirect Automatic"),
+                                P_("If composite window redirects are set to "
+                                   "Automatic (or Manual if false)"),
                                 TRUE,
                                 G_PARAM_READWRITE);
 
@@ -723,8 +572,8 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
 
 
   pspec = g_param_spec_boolean ("window-mapped",
-                                "Window Mapped",
-                                "If window is mapped",
+                                P_("Window Mapped"),
+                                P_("If window is mapped"),
                                 FALSE,
                                 G_PARAM_READABLE);
 
@@ -733,8 +582,8 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
 
 
   pspec = g_param_spec_boolean ("destroyed",
-                                "Destroyed",
-                                "If window has been destroyed",
+                                P_("Destroyed"),
+                                P_("If window has been destroyed"),
                                 FALSE,
                                 G_PARAM_READABLE);
 
@@ -742,8 +591,8 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
                                    PROP_DESTROYED, pspec);
 
   pspec = g_param_spec_int ("window-x",
-                            "Window X",
-                            "X position of window on screen according to X11",
+                            P_("Window X"),
+                            P_("X position of window on screen according to X11"),
                             G_MININT, G_MAXINT, 0, G_PARAM_READABLE);
 
   g_object_class_install_property (object_class,
@@ -751,16 +600,16 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
 
 
   pspec = g_param_spec_int ("window-y",
-                            "Window Y",
-                            "Y position of window on screen according to X11",
+                            P_("Window Y"),
+                            P_("Y position of window on screen according to X11"),
                             G_MININT, G_MAXINT, 0, G_PARAM_READABLE);
 
   g_object_class_install_property (object_class,
                                    PROP_WINDOW_Y, pspec);
 
   pspec = g_param_spec_boolean ("window-override-redirect",
-                                "Window Override Redirect",
-                                "If this is an override-redirect window",
+                                P_("Window Override Redirect"),
+                                P_("If this is an override-redirect window"),
                                 FALSE,
                                 G_PARAM_READABLE);
 
@@ -778,13 +627,13 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
    * Since: 0.8
    */
   signals[UPDATE_AREA] =
-      g_signal_new ("update-area",
+      g_signal_new (g_intern_static_string ("update-area"),
                     G_TYPE_FROM_CLASS (object_class),
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET (ClutterX11TexturePixmapClass, \
                                      update_area),
                     NULL, NULL,
-                    clutter_marshal_VOID__INT_INT_INT_INT,
+                    _clutter_marshal_VOID__INT_INT_INT_INT,
                     G_TYPE_NONE, 4,
                     G_TYPE_INT,
                     G_TYPE_INT,
@@ -799,9 +648,11 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
    * @width: The width of the damage region
    * @height: The height of the damage region
    *
-   * ::queue-damage-redraw is emitted to notify that some sub-region of the
-   * underlying pixmap has changed and you need to queue a
-   * corresponding redraw for the actor.
+   * ::queue-damage-redraw is emitted to notify that some sub-region
+   * of the texture has been changed (either by an automatic damage
+   * update or by an explicit call to
+   * clutter_x11_texture_pixmap_update_area). This usually means a
+   * redraw needs to be queued for the actor.
    *
    * The default handler will queue a clipped redraw in response to
    * the damage, using the assumption that the pixmap is being painted
@@ -818,24 +669,22 @@ clutter_x11_texture_pixmap_class_init (ClutterX11TexturePixmapClass *klass)
                   G_SIGNAL_RUN_FIRST,
                   0,
                   NULL, NULL,
-                  clutter_marshal_VOID__INT_INT_INT_INT,
+                  _clutter_marshal_VOID__INT_INT_INT_INT,
                   G_TYPE_NONE, 4,
                   G_TYPE_INT,
                   G_TYPE_INT,
                   G_TYPE_INT,
                   G_TYPE_INT);
 
-  g_signal_override_class_closure (signals[QUEUE_DAMAGE_REDRAW],
+  g_signal_override_class_handler ("queue-damage-redraw",
                                    CLUTTER_X11_TYPE_TEXTURE_PIXMAP,
-                                   g_cclosure_new ((GCallback) clutter_x11_texture_pixmap_real_queue_damage_redraw,
-                                                   NULL,
-                                                   NULL));
+                                   G_CALLBACK (clutter_x11_texture_pixmap_real_queue_damage_redraw));
 
   default_backend = clutter_get_default_backend ();
 
   if (!CLUTTER_IS_BACKEND_X11 (default_backend))
     {
-      g_critical ("ClutterX11TexturePixmap instanciated with a "
+      g_critical ("ClutterX11TexturePixmap instantiated with a "
                   "non-X11 backend");
       return;
     }
@@ -848,196 +697,12 @@ clutter_x11_texture_pixmap_update_area_real (ClutterX11TexturePixmap *texture,
                                              gint                     width,
                                              gint                     height)
 {
-  ClutterX11TexturePixmapPrivate       *priv;
-  Display                              *dpy;
-  XImage                               *image;
-  char				       *first_pixel;
-  GError                               *error = NULL;
-  guint                                 bytes_per_line;
-  char				       *data;
-  gboolean                              data_allocated = FALSE;
-  int                                   err_code;
+  CoglHandle cogl_texture;
 
-  if (!CLUTTER_ACTOR_IS_REALIZED (texture))
-    return;
+  cogl_texture = clutter_texture_get_cogl_texture (CLUTTER_TEXTURE (texture));
 
-  priv = texture->priv;
-  dpy  = clutter_x11_get_default_display();
-
-  if (!priv->pixmap)
-    return;
-
-  if (priv->shminfo.shmid == -1)
-    try_alloc_shm (texture);
-
-  clutter_x11_trap_x_errors ();
-
-  if (priv->have_shm)
-    {
-      image =
-	XShmCreateImage(dpy,
-			DefaultVisual(dpy,
-				      clutter_x11_get_default_screen()),
-			priv->depth,
-			ZPixmap,
-			NULL,
-			&priv->shminfo,
-			width,
-			height);
-      image->data = priv->shminfo.shmaddr;
-
-      XShmGetImage (dpy, priv->pixmap, image, x, y, AllPlanes);
-      first_pixel = image->data;
-    }
-  else
-    {
-      if (!priv->image)
-	{
-          priv->image = XGetImage (dpy,
-                                   priv->pixmap,
-                                   0, 0,
-                                   priv->pixmap_width, priv->pixmap_height,
-                                   AllPlanes,
-                                   ZPixmap);
-	  first_pixel  = priv->image->data + priv->image->bytes_per_line * y
-			  + x * priv->image->bits_per_pixel/8;
-	}
-      else
-	{
-          XGetSubImage (dpy,
-                        priv->pixmap,
-                        x, y,
-                        width, height,
-                        AllPlanes,
-                        ZPixmap,
-                        priv->image,
-                        x, y);
-	  first_pixel  = priv->image->data + priv->image->bytes_per_line * y
-            + x * priv->image->bits_per_pixel/8;
-	}
-      image = priv->image;
-    }
-
-  XSync (dpy, FALSE);
-
-  if ((err_code = clutter_x11_untrap_x_errors ()))
-    {
-      g_warning ("Failed to get XImage of pixmap: %lx, removing",
-                 priv->pixmap);
-      /* safe to assume pixmap has gone away? - therefor reset */
-      clutter_x11_texture_pixmap_set_pixmap (texture, None);
-      return;
-    }
-
-  if (priv->depth == 24)
-    {
-      guint xpos, ypos;
-
-      for (ypos=0; ypos<height; ypos++)
-	for (xpos=0; xpos<width; xpos++)
-            {
-	    char *p = first_pixel + image->bytes_per_line*ypos
-			  + xpos * 4;
-	    p[3] = 0xFF;
-        }
-
-      data = first_pixel;
-      bytes_per_line = image->bytes_per_line;
-    }
-  else if (priv->depth == 16)
-    {
-      guint xpos, ypos;
-      data = g_malloc (height * width * 4);
-      data_allocated = TRUE;
-      bytes_per_line = width * 4;
-
-      for (ypos=0; ypos<height; ypos++)
-	for (xpos=0; xpos<width; xpos++)
-            {
-	    char *src_p = first_pixel + image->bytes_per_line * ypos
-			    + xpos * 2;
-	    guint16 *src_pixel = (guint16 *)src_p;
-	    char *dst_p = data + bytes_per_line * ypos + xpos * 4;
-	    guint32 *dst_pixel = (guint32 *)dst_p;
-
-	    *dst_pixel =
-	      ((((*src_pixel << 3) & 0xf8) | ((*src_pixel >> 2) & 0x7)) | \
-	      (((*src_pixel << 5) & 0xfc00) | ((*src_pixel >> 1) & 0x300)) | \
-	      (((*src_pixel << 8) & 0xf80000) | ((*src_pixel << 3) & 0x70000)))
-	      | 0xff000000;
-	  }
-    }
-  else if (priv->depth == 32)
-    {
-      bytes_per_line = image->bytes_per_line;
-      data = first_pixel;
-    }
-  else
-    return;
-
-  /* For debugging purposes, un comment to simply generate dummy
-   * pixmap data. (A Green background and Blue cross) */
-#if 0
-  {
-    guint xpos, ypos;
-
-    if (data_allocated)
-      g_free (data);
-    data_allocated = TRUE;
-    data = g_malloc (width*height*4);
-    bytes_per_line = width *4;
-
-    for (ypos=0; ypos<height; ypos++)
-      for (xpos=0; xpos<width; xpos++)
-	{
-	  char *p = data + width*4*ypos + xpos * 4;
-	  guint32 *pixel = (guint32 *)p;
-	  if ((xpos > width/2 && xpos <= (width/2) + width/4)
-	      || (ypos > height/2 && ypos <= (height/2) + height/4))
-	    *pixel=0xff0000ff;
-	  else
-	    *pixel=0xff00ff00;
-	}
-  }
-#endif
-
-  if (x != 0 || y != 0 ||
-      width != priv->pixmap_width || height != priv->pixmap_height)
-    clutter_texture_set_area_from_rgb_data  (CLUTTER_TEXTURE (texture),
-					     (guint8 *)data,
-					     TRUE,
-					     x, y,
-					     width, height,
-					     bytes_per_line,
-					     4,
-					     CLUTTER_TEXTURE_RGB_FLAG_BGR |
-					     CLUTTER_TEXTURE_RGB_FLAG_PREMULT,
-					     &error);
-  else
-    clutter_texture_set_from_rgb_data  (CLUTTER_TEXTURE (texture),
-					(guint8 *)data,
-					TRUE,
-					width, height,
-					bytes_per_line,
-					4,
-					CLUTTER_TEXTURE_RGB_FLAG_BGR |
-					CLUTTER_TEXTURE_RGB_FLAG_PREMULT,
-					&error);
-
-
-
-  if (error)
-    {
-      g_warning ("Error when uploading from pixbuf: %s",
-                 error->message);
-      g_error_free (error);
-    }
-
-  if (data_allocated)
-    g_free (data);
-
-  if (priv->have_shm)
-    XFree (image);
+  if (cogl_texture)
+    cogl_texture_pixmap_x11_update_area (cogl_texture, x, y, width, height);
 }
 
 /**
@@ -1123,6 +788,8 @@ clutter_x11_texture_pixmap_set_pixmap (ClutterX11TexturePixmap *texture,
   Status       status = 0;
   gboolean     new_pixmap = FALSE, new_pixmap_width = FALSE;
   gboolean     new_pixmap_height = FALSE, new_pixmap_depth = FALSE;
+  CoglHandle   material;
+  CoglHandle   cogl_texture;
 
   ClutterX11TexturePixmapPrivate *priv;
 
@@ -1130,30 +797,35 @@ clutter_x11_texture_pixmap_set_pixmap (ClutterX11TexturePixmap *texture,
 
   priv = texture->priv;
 
-  clutter_x11_trap_x_errors ();
+  /* Get rid of the existing Cogl texture early because it may try to
+     use the pixmap which we might destroy */
+  material = clutter_texture_get_cogl_material (CLUTTER_TEXTURE (texture));
+  if (material)
+    cogl_material_set_layer (material, 0, COGL_INVALID_HANDLE);
 
-  status = XGetGeometry (clutter_x11_get_default_display(),
-                         (Drawable)pixmap,
-                         &root,
-                         &x,
-                         &y,
-                         &width,
-                         &height,
-                         &border_width,
-                         &depth);
-
-  if (clutter_x11_untrap_x_errors () || status == 0)
+  if (pixmap != None)
     {
-      if (pixmap != None)
-        g_warning ("Unable to query pixmap: %lx", pixmap);
-      pixmap = None;
-      width = height = depth = 0;
+      clutter_x11_trap_x_errors ();
+      status = XGetGeometry (clutter_x11_get_default_display(),
+			     (Drawable)pixmap,
+			     &root,
+			     &x,
+			     &y,
+			     &width,
+			     &height,
+			     &border_width,
+			     &depth);
+
+      if (clutter_x11_untrap_x_errors () || status == 0)
+	{
+	  g_warning ("Unable to query pixmap: %lx", pixmap);
+	  pixmap = None;
+	  width = height = depth = 0;
+	}
     }
-
-  if (priv->image)
+  else
     {
-      XDestroyImage (priv->image);
-      priv->image = NULL;
+      width = height = depth = 0;
     }
 
   if (priv->pixmap != pixmap)
@@ -1164,12 +836,10 @@ clutter_x11_texture_pixmap_set_pixmap (ClutterX11TexturePixmap *texture,
       priv->pixmap = pixmap;
       new_pixmap = TRUE;
 
-      /* The damage object is created on the window if there is any
-       * but if there is no window, then we create it directly on
-       * the pixmap, so it needs to be recreated with a change in
-       * pixmap.
+      /* The damage object is created on the pixmap, so it needs to be
+       * recreated with a change in pixmap.
        */
-      if (priv->automatic_updates && new_pixmap && !priv->window)
+      if (priv->automatic_updates)
 	{
 	  free_damage_resources (texture);
 	  create_damage_resources (texture);
@@ -1198,6 +868,7 @@ clutter_x11_texture_pixmap_set_pixmap (ClutterX11TexturePixmap *texture,
    * above members so the values are all available to the
    * signal handlers. */
   g_object_ref (texture);
+
   if (new_pixmap)
     g_object_notify (G_OBJECT (texture), "pixmap");
   if (new_pixmap_width)
@@ -1207,19 +878,13 @@ clutter_x11_texture_pixmap_set_pixmap (ClutterX11TexturePixmap *texture,
   if (new_pixmap_depth)
     g_object_notify (G_OBJECT (texture), "pixmap-depth");
 
-  free_shm_resources (texture);
-
-  if (priv->depth != 0 &&
-      priv->pixmap != None &&
-      priv->pixmap_width != 0 &&
-      priv->pixmap_height != 0)
+  if (pixmap)
     {
-      if (CLUTTER_ACTOR_IS_REALIZED (texture))
-        clutter_x11_texture_pixmap_update_area (texture,
-                                                0, 0,
-                                                priv->pixmap_width,
-                                                priv->pixmap_height);
-
+      cogl_texture = cogl_texture_pixmap_x11_new (pixmap, FALSE);
+      clutter_texture_set_cogl_texture (CLUTTER_TEXTURE (texture),
+                                        cogl_texture);
+      cogl_handle_unref (cogl_texture);
+      update_pixmap_damage_object (texture);
     }
 
   /*
@@ -1274,6 +939,8 @@ clutter_x11_texture_pixmap_set_window (ClutterX11TexturePixmap *texture,
                                   CompositeRedirectAutomatic : CompositeRedirectManual);
       XSync (clutter_x11_get_default_display (), False);
       clutter_x11_untrap_x_errors ();
+
+      clutter_x11_texture_pixmap_set_pixmap (texture, None);
     }
 
   priv->window = window;
@@ -1312,12 +979,6 @@ clutter_x11_texture_pixmap_set_window (ClutterX11TexturePixmap *texture,
       clutter_x11_add_filter (on_x_event_filter_too, (gpointer)texture);
     }
 
-  if (priv->automatic_updates)
-    {
-      free_damage_resources (texture);
-      create_damage_resources (texture);
-    }
-
   g_object_ref (texture);
   g_object_notify (G_OBJECT (texture), "window");
 
@@ -1500,6 +1161,7 @@ clutter_x11_texture_pixmap_set_automatic (ClutterX11TexturePixmap *texture,
 
   priv = texture->priv;
 
+  setting = !!setting;
   if (setting == priv->automatic_updates)
     return;
 
@@ -1511,6 +1173,4 @@ clutter_x11_texture_pixmap_set_automatic (ClutterX11TexturePixmap *texture,
     free_damage_resources (texture);
 
   priv->automatic_updates = setting;
-
 }
-
